%This code creates a bivariate file of peaks and troughs from BB results
%Maximo Camacho, Lola Gadea 2016
%The procedure has the following steps
%First, we decide if the economy stars with a peak or a trough
%Second, we adjust the pairs by removing the peak or trough over
%Third, we arrange the resulted pairs maintaining the peak/trough as reference
%INPUTS: index of peaks, index of troughs, dates of the sample
%OUTPUT: pair of peaks/troughs or troughs/peaks, start returns P if the economy starts by a peak and T if starts by a trough

function [PAIR, start]=preparing_file_bi(peaks,troughs,dates)

[t,n]=size(peaks);
%Firtsly, we decide if the economy starts with a peak or a trough
[first,ratio]=selecting_first(peaks,troughs);
if first=='P'
    start=1;
elseif first=='T'
    start=2;
end
switch first
    case 'P'
ii=1;
PAIR=[];
for j=1:n
ser_p=peaks(:,j);
ser_t=troughs(:,j);
index_p=find(ser_p==1);
index_t=find(ser_t==1);
if isempty(index_p)==0 &&  isempty(index_t)==0 
%remove the first trough of the series if the series starts by trough
if index_t(1)<index_p(1) index_t=index_t(2:end); end 
if isempty(index_t)==0 
    if size(index_p,1)==size(index_t,1) %adjust the size of both series
        l=size(index_p,1);
    else l=size(index_p,1)-1;
    end
for i=1:l
    pair(1,1)=index_p(i);
    pair(2,1)=index_t(i);
    PAIR(ii,:)=pair;
    ii=ii+1;
end
end
end
end


case 'T'
ii=1;
PAIR=[];
for j=1:n
ser_p=peaks(:,j);
ser_t=troughs(:,j);
index_p=find(ser_p==1);
index_t=find(ser_t==1);
if isempty(index_p)==0 &&  isempty(index_t)==0 
%remove the first peak of the series if the series starts by peak
if index_t(1)>index_p(1) index_p=index_p(2:end); end
if isempty(index_p)==0 
    if size(index_t,1)==size(index_p,1)  %adjust the size of both series
         l=size(index_t,1);
    else l=size(index_t,1)-1;
    end
for i=1:l
    pair(1,1)=index_t(i);
    pair(2,1)=index_p(i);
    PAIR(ii,:)=pair;
    ii=ii+1;
end
end
end
end
end


PAIR=sort_matrix_bi(PAIR); %arraging pairs
PAIR=dates(PAIR);
PAIR=months_to_decimals_matrix(PAIR);

%%%%%%%%%%%%%%%%%%%%%%%%%%AUXILIARY%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function B=sort_matrix_ascend(A,n)
a=A(:,n);
[value,index]=sort(a);
B=A(index,:);

function [x, ratio]=selecting_first(peaks,troughs)
[t,n]=size(troughs);
p=0; t=0;
for j=1:n
ser_p=peaks(:,j);
ser_t=troughs(:,j);
index_p=find(ser_p==1);
index_t=find(ser_t==1);
if isempty(index_p)==0 &&  isempty(index_t)==0 
if index_p(1)<index_t(1);
    p=p+1;
else
    t=t+1;
end
end
end

if p>t x='P';
else
    x='T';
end
ratio=p/n;

 function M2=months_to_decimals_mat(M)
[t,n]=size(M);
M2=NaN(t,n);
for i=1:n
for k=1:t
    m=M(k,i);
    dec_m=m-floor(m);
    for j=1:12
    if dec_m>0.01*(j-1) && dec_m<=0.01*j
        m2=floor(m)+1/12*(j-1);
    end
    end
    M2(k,i)=m2;
end
end

function C=sort_matrix_bi(A)
A=sort_matrix_ascend(A,1);
C=A;
t=size(A,1);
i=1; j=0; n=0; l=0; 
while (i<t+1)
    if i==1 n=A(i,1); end
    if n~=A(i,1) n=A(i,1);
        if j~=0 l=i-1; end
    else
    if j==0 && i~=1 j=i-1; end
    end
    if j~=0 && l~=0
        B=new_submatrix(A,l,j);
        B=sort_matrix_ascend(B,1);
        C=reform_matrix(C,B,j,l);
        j=0; l=0;
    end
i=i+1;
end


function B=new_submatrix(A,l,j)
    u=j; v=l; w=1; 
    while (u<=v)
        B(w,1)=A(u,2);
        B(w,2)=A(u,1);
        B(w,3)=u;
        w=w+1; u=u+1;
    end


function C=reform_matrix(C,B,j,l)
z=1; x=j;
while (z<=l-j+1)
    C(x,1)=B(z,2);
    C(x,2)=B(z,1);
    x=x+1;
    z=z+1;
end
