% This code reproduces the empirical illustration of 
% "A new approach to dating the reference cycle"
% by Maximo Camacho, Lola Gadea, and Ana Gomez-Loscos (2018)
% It develops a model selection based on Log-lokelihood, AIC, BIC, Entropy, BIC+Entropy and Bayes factor
% The results could be slightly different from ours if the version of Matlab is not 2016a

clear
matlabrc; % Clear previous path and restore the original one
clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 Please, adjust the paths                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
p1 = genpath('C:\aa\Empirical\functions_empirical');
path(path,p1) %general auxiliary matlab functions
p2 = genpath('C:\aa\Empirical\general_functions');
path(path,p2) %specific matlab functions of the empirical section

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Load turning point data and adjust control specifications   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load Y_NBER.dat
Y=Y_NBER;
kmin=1; kmax=9;
K=kmin:kmax;
LOGLIK=NaN(length(K),1);
AIC=NaN(length(K),1);
BIC=NaN(length(K),1);
Entropy=NaN(length(K),1);
BIC_EN=NaN(length(K),1);
W1=NaN(length(K),1);
W2=NaN(length(K),1);
W3=NaN(length(K),1);
start=1; %the economy starts by peak (2, if starts by trough)


for k=1:length(K)
    k
    if k==1        
mcmcout=estimate_K1_bi_uniformative(Y);
    else
mcmcout=estimate_MS_K_bi_uniformative(Y,k,start);
    end
LOGLIK(k)=mcmcout.ic.loglikmax;
AIC(k)=mcmcout.ic.aic;
BIC(k)=mcmcout.ic.bic;
Entropy(k)=mcmcout.ic.entropy_ml;
BIC_EN(k)=mcmcout.ic.iclbic_ml;
if k>1
W1(k)=mcmcout.warning.one;
W2(k)=mcmcout.warning.two;
W3(k)=mcmcout.warning.three;
end
end

%Compute bayes factor
bayes_factor=NaN(kmax,kmax);
for i=1:kmax
    for j=i+1:kmax
        bayes_factor(j,i)=BIC(i)-BIC(j);
    end
end

for i=1:kmax-1
    BF(i)=bayes_factor(i+1,i);
end

% xbars={'K=1/K=2','K=2/K=3','K=3/K=4','K=4/K=5','K=5/K=6','K=6/K=7','K=7/K=8','K=8/K=9'};
% bar(BF)
% ylabel('Bayes factor');
% set(gca,'xtick',1:1:kmax-1);
% set(gca,'xticklabel',xbars);


IC=[LOGLIK,AIC,BIC,Entropy,BIC_EN];
IC=[IC,[NaN;BF']];

%Make Table
MEASURES={'& LogLik','& AIC','& BIC','& Entropy','& BIC-Entropy','& Bayes factor (k=i/k=i+1)'};

file=strcat('Table_selectK', '.tex');
fid=fopen(file,'w');
tt=['\\begin{table}\\caption{Results of select K}\\label{tab:1}\\begin{center}\\scalebox{1}{\\begin{tabular}{l|' ... 
        '*{' int2str(6) '}{c}} \\hline \\hline \n'];
fprintf(fid,tt);
tt=['K ' '& LogLik','& AIC','& BIC','& Entropy','& BIC-Entropy','& Bayes factor (k=i/k=i+1)' '\\\\ \\hline \n']; 
fprintf(fid,tt);

for i=1:size(IC,1)
    tt=[];
    for j=1:size(MEASURES,2)
    tt=[tt,'&' ,num2str(IC(i,j),'%5.2f') ];
    end   
    tt=[num2str(i),tt,'\\\\ \n'];
    fprintf(fid,tt);
end

tt='\\hline \\hline \\end{tabular}}\\end{center}\\end{table}\n';
fprintf(fid,tt);
st=fclose(fid);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    Print the main results                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc;
disp('Table 3: Model selection');
disp('     Log-likelihood     AIC          BIC          Entropy     BIC+Entropy   Bayes factor');
disp(IC);



